//---------------------------------------------------//
//  HC-SR04 demo program
//  Version 1.0 2018.6.12 T.Hoshi
//---------------------------------------------------//
#include <Wire.h>
// HC-SR04のピン接続設定
#define echoPin 3 // Echo Pin
#define trigPin 2 // Trigger Pin
// I2Cのアドレス設定
#define LCD_ADRS  0x3E //液晶表示器
#define TH_Pin 0 //103ATサーミスタ 
//計算用変数
double duration ; //反射時間μs
float distance ; //距離
float mv ; // サーミスタ計測電圧
float temperature ; // 温度
//LCD表示用バッファ
char lcd_buf[17] ;
int i ;

//LCDにデータを送るサブルーチン
void LCD_write(char t_data[]) {
  for(i = 0; i < 16; i++) {
    Wire.beginTransmission(LCD_ADRS);
    Wire.write(0x40);
    Wire.write(t_data[i]);
    Wire.endTransmission();
    delay(1);
  }
} 

//LCDに命令を実行させるサブルーチン
void LCD_command(byte t_command)
{
  Wire.beginTransmission(LCD_ADRS);
  Wire.write(0x00);
  Wire.write(t_command);
  Wire.endTransmission();
  delay(10);
}

void setup() {
  Wire.begin();   // I2C初期化
  //LCD初期化
  LCD_command(0x38);  LCD_command(0x39);  LCD_command(0x14);
  LCD_command(0x73);  LCD_command(0x52);  LCD_command(0x6C);
  LCD_command(0x38);  LCD_command(0x01);  LCD_command(0x0C);
  LCD_command(0x80|0x00); // 1行目タイトル表示 
  LCD_write("* HC-SR04 TEST *"); 
  //距離センサ信号線の入出力設定
  pinMode( echoPin, INPUT );
  pinMode( trigPin, OUTPUT );
}

void loop() {
  //気温測定
  mv = (float)analogRead(TH_Pin) *  5 / 1024 ;
  if((mv > 4.8)||(mv < 0.9)) { // temp. zone  -19,21 C to +64.37 C
    temperature = 15.0 ; //温度計測不能なので音速340 m/sにセット
  }
  else {
     temperature = (1.3679 * mv - 29.229) * mv + 89.57 ; //温度換算
  }  
  //距離計測
  digitalWrite(trigPin, LOW); 
  delayMicroseconds(2); 
  digitalWrite( trigPin, HIGH ); //:計測命令を出力
  delayMicroseconds( 10 ); //
  //超音波発射
  digitalWrite( trigPin, LOW );
  //反射時間計測
  duration = pulseIn( echoPin, HIGH ); //センサからの入力
  if (duration < 0) distance = 0.0 ; //エラー時は0に
  else {
    duration = duration / 2 ; //往復距離を半分にする
    //距離をcm単位で計算
    distance = (float)duration * (331.5 + 0.6 * temperature) * 100 / 1000000 ;
  }
  //LCD表示値セット
  for(i=0 ; i<16 ; i++) lcd_buf[i]=' ' ; // 表示バッファクリア
  dtostrf(distance,6,2,&lcd_buf[0]) ; // 距離
  lcd_buf[6]= 'c' ;  lcd_buf[7]= 'm' ; // cm表示
  lcd_buf[8]= ' ' ;
  dtostrf(temperature,5,1,&lcd_buf[9]) ; // 気温
  lcd_buf[14]= 0xf2 ; lcd_buf[15]= 'C' ; // ℃表示
  //LCDに測定値表示
  LCD_command(0x80|0x40);
  LCD_write(lcd_buf) ;
  delay(800) ;
}
